import { PowerShell, PowerShellOptions } from 'node-powershell';
import { DefaultShim } from '../powershell/shim/DefaultShim';
import { Shim } from '../powershell/shim/Shim';
/**
 * base interface for all powershell binding classes
 */
export interface IPowerShellBinding {
    /**
     * get the powershell instance of this binding
     */
    get shell(): PowerShell;
    /**
     * get the shim instance to use for commands in this binding
     */
    get shim(): Shim;
}
/**
 * options for {@link PowerShellBinding}
 */
export interface PowerShellBindingOptions extends PowerShellOptions {
    /**
     * pre-initialized shell instance to use for the binding.
     * if defined, all other options are ignored
     */
    instance?: PowerShell;
}
/**
 * default powershell binding class implementation
 */
export declare abstract class PowerShellBinding implements IPowerShellBinding {
    private readonly _shell;
    private readonly _shim;
    constructor(options?: PowerShellBindingOptions);
    get shell(): PowerShell;
    get shim(): DefaultShim;
}
