import { TypeGuard } from '@shadow578/type-guardian';
import { Shim } from '../powershell/shim/Shim';
import { IPowerShellBinding } from './PowerShellBinding';
/**
 * additional options for {@link PowerShellDecorator}
 */
export interface PowerShellDecoratorOptions {
    /**
     * expand parameters to be available directly.
     * if enabled, '$params.MyParameter' can be replaced with '$MyParameter'.
     *
     * default value: true
     */
    expandParameters: boolean;
    /**
     * how deep the output object will be serialized.
     * note that higher values may impact performance.
     *
     * default value: 2
     */
    serializationDepth: number;
    /**
     * override for the shim used with this command.
     * if not defined, the shim provided by the binding class is used.
     *
     * default value: undefined
     */
    shim?: Shim;
}
/**
 * shorthand for {@link PowerShellCall}
 */
export declare const PS: typeof PowerShellCall;
/**
 * decorator for powershell binding functions
 *
 * @param command the powershell command to be executed by this binding function
 * @param typeGuard type guard to validate the result data parsed from powershell
 * @param options additional options for the binding
 * @example
 * class ExampleBinding extends PowerShellBinding {
 *     \@PowerShell('Write-Output "Hello, $name"', isString)
 *     greet(name: string): Promise<string> {
 *         // dummy implementation
 *         return <Promise<string>>{}
 *     }
 * }
 */
export declare function PowerShellCall<T = void>(command: string, typeGuard?: TypeGuard<T>, options?: Partial<PowerShellDecoratorOptions>): (_target: IPowerShellBinding, _methodName: string, descriptor: TypedPropertyDescriptor<(...params: any[]) => Promise<T>>) => void;
