"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PowerShellCall = exports.PS = void 0;
var TypeGuards_1 = require("@shadow578/type-guardian/lib/TypeGuards");
var Parameters_1 = require("../powershell/shim/Parameters");
var util_1 = require("../util");
/**
 * default values for {@link PowerShellDecoratorOptions}
 */
var DEFAULT_OPTIONS = {
    expandParameters: true,
    serializationDepth: 2,
};
/**
 * shorthand for {@link PowerShellCall}
 */
//TODO move to index.ts
exports.PS = PowerShellCall;
/**
 * decorator for powershell binding functions
 *
 * @param command the powershell command to be executed by this binding function
 * @param typeGuard type guard to validate the result data parsed from powershell
 * @param options additional options for the binding
 * @example
 * class ExampleBinding extends PowerShellBinding {
 *     \@PowerShell('Write-Output "Hello, $name"', isString)
 *     greet(name: string): Promise<string> {
 *         // dummy implementation
 *         return <Promise<string>>{}
 *     }
 * }
 */
function PowerShellCall(command, typeGuard, options) {
    if (typeGuard === void 0) { typeGuard = TypeGuards_1.isAny; }
    if (options === void 0) { options = {}; }
    // merge default values into provided options
    var fullOptions = __assign(__assign({}, DEFAULT_OPTIONS), options);
    // build the decorator
    return function (_target, _methodName, descriptor) {
        // get parameter name list of original method
        var originalMethod = descriptor.value;
        if (originalMethod === undefined) {
            throw new TypeError('method descriptor value was undefined');
        }
        var parameterNames = (0, util_1.getMethodParameterNames)(originalMethod);
        // re-defined the method
        descriptor.value = function () {
            var _a, _b;
            var parameterValues = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                parameterValues[_i] = arguments[_i];
            }
            return __awaiter(this, void 0, void 0, function () {
                var executionId, shim, parameters, shimmedCommand, invocationResult, result, _c, _d, element;
                return __generator(this, function (_e) {
                    switch (_e.label) {
                        case 0:
                            executionId = (0, util_1.getRandomString)(20);
                            shim = fullOptions.shim || this.shim;
                            parameters = makeParameterRecord(parameterNames, parameterValues);
                            shimmedCommand = shim.prepare(executionId, command, parameters, {
                                expandParameters: fullOptions.expandParameters,
                                resultSerializationDepth: fullOptions.serializationDepth,
                            });
                            return [4 /*yield*/, this.shell.invoke(shimmedCommand)];
                        case 1:
                            invocationResult = _e.sent();
                            result = shim.parseResult(executionId, shim.requiresStdout ? (_a = invocationResult.stdout) === null || _a === void 0 ? void 0 : _a.toString() : undefined, shim.requiresStderr ? (_b = invocationResult.stderr) === null || _b === void 0 ? void 0 : _b.toString() : undefined);
                            // re-throw error thrown in poweshell
                            if (result.error !== undefined) {
                                throw result.error;
                            }
                            // attempt to validate the result data as-is
                            if (typeGuard(result.data)) {
                                return [2 /*return*/, result.data];
                            }
                            // if the result data is a array, attempt to
                            // validate the elements in result data one-by-one
                            if (Array.isArray(result.data)) {
                                for (_c = 0, _d = result.data; _c < _d.length; _c++) {
                                    element = _d[_c];
                                    if (typeGuard(element)) {
                                        return [2 /*return*/, element];
                                    }
                                }
                            }
                            // validation failed for all possible values
                            throw new TypeError('validation of result data failed');
                    }
                });
            });
        };
    };
}
exports.PowerShellCall = PowerShellCall;
/**
 * convert a method's raw parameter list to a record from parameter name to parameter value
 *
 * @param parameterNames the parameter name list. see {@link getMethodParameterNames}
 * @param parameterValues the parameter value list
 * @returns a record from the method parameter name to the parameter value
 */
function makeParameterRecord(parameterNames, parameterValues) {
    // ensure parameter names and parameter values have same length
    if (parameterNames.length !== parameterValues.length) {
        throw new RangeError('parameter values length is unequal to expected parameter names length');
    }
    // create a map from parameter name to parameter value
    var parameters = {};
    parameterNames.forEach(function (name, i) {
        parameters[name] = parameterValues[i];
    });
    (0, Parameters_1.assertParametersValid)(parameters);
    return parameters;
}
//# sourceMappingURL=PowerShellDecorator.js.map