"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultShim = void 0;
var type_guardian_1 = require("@shadow578/type-guardian");
var util_1 = require("../../util");
var PowerShellException_1 = require("../PowerShellException");
var TypeGuards_1 = require("@shadow578/type-guardian/lib/TypeGuards");
/**
 * type guard for {@link ShimOutput}
 */
var isShimOutput = (0, type_guardian_1.matchObject)({
    success: 'boolean',
    data: (0, type_guardian_1.matchArray)('any'),
});
var DefaultShim = /** @class */ (function () {
    function DefaultShim() {
    }
    Object.defineProperty(DefaultShim.prototype, "requiresStdout", {
        get: function () {
            return true;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DefaultShim.prototype, "requiresStderr", {
        get: function () {
            return false;
        },
        enumerable: false,
        configurable: true
    });
    DefaultShim.prototype.prepare = function (id, command, parameters, options) {
        // convert the json to a utf-8 encoded base64 string
        // doing the transfer in base64 ensures that we don't run into issues
        // in case wierd characters are used in the parameter values
        var paramsBase64String = (0, util_1.base64Encode)(JSON.stringify(parameters));
        // build the shim
        //prettier-ignore
        return "function __shim() {\n        .{\n            function __target() {\n                ".concat(command, "\n            }\n            try {\n                $params = ([System.Text.Encoding]::UTF8.GetString([System.Convert]::FromBase64String(\"").concat(paramsBase64String, "\")) | ConvertFrom-Json)\n                ").concat(
        // parameter expansion
        options.expandParameters
            ? '$params.PSObject.Properties | ForEach-Object { Set-Variable -Name $_.Name -Value $_.Value }'
            : '', "\n                $__capture = @{\n                    \"success\" = $true\n                    \"data\"    = @(__target)\n                }\n            }\n            catch {\n                $__capture = @{\n                    \"success\" = $false\n                    \"data\"    = @(@{\n                      \"Exception\" = ($_.Exception.Message)\n                      \"FullyQualifiedErrorId\" = ($_.FullyQualifiedErrorId)\n                      \"InvocationInfo\" = (\"$($_.InvocationInfo.Line) (at $($_.InvocationInfo.ScriptLineNumber),$($_.InvocationInfo.OffsetInLine) )\")\n                    })\n                }\n            }\n        } | Out-Null\n        Write-Output \"{{").concat(id, "=$(([System.Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes((ConvertTo-Json $__capture -Depth (").concat(Math.min(options.resultSerializationDepth + 1, 100), ") -Compress)))))}}\"\n    }; __shim");
    };
    DefaultShim.prototype.parseResult = function (id, stdout, _stderr) {
        if (!(0, TypeGuards_1.isString)(stdout)) {
            throw new TypeError('stdout was not defined');
        }
        // find the result data in the output stream
        var resultMatch = stdout.match(new RegExp("{{(?:".concat(id, ")=([A-Za-z0-9+/-_]*={0,3})}}")));
        if (!resultMatch) {
            throw new Error('could not find result data in output stream');
        }
        // base64-decode and parse the result
        var outputJson = (0, util_1.base64Decode)(resultMatch[1]);
        var output = JSON.parse(outputJson);
        // check result
        if (!isShimOutput(output)) {
            throw new TypeError('could not decode shim output');
        }
        // transform output
        if (output.success) {
            // if output data is zero length, the command returned nothing (void call)
            // so we have to return undefined in data
            if (output.data.length === 0) {
                return {
                    data: undefined,
                };
            }
            else {
                return {
                    data: output.data,
                };
            }
        }
        else {
            if (!(0, PowerShellException_1.isPowerShellException)(output.data[0])) {
                throw new TypeError('could not decode exception from shim output');
            }
            return {
                error: output.data[0],
            };
        }
    };
    return DefaultShim;
}());
exports.DefaultShim = DefaultShim;
//# sourceMappingURL=DefaultShim.js.map