"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertParametersValid = void 0;
var type_guardian_1 = require("@shadow578/type-guardian");
/**
 * type guard for {@link ParameterValue}
 */
var isParameterValue = (0, type_guardian_1.matchUnion)('string', 'number', 'boolean', 'object');
/**
 * check if a parameter map only contains values for supported types.
 * if a value is found that is not valid, a error is thrown detailing the offending key and value type.
 *
 * @param parameters the parameters to validate
 */
function assertParametersValid(parameters) {
    var _loop_1 = function (key) {
        var value = parameters[key];
        // skip undefined and null values
        if (value === undefined || value === null) {
            return "continue";
        }
        // check if value is a array
        if (Array.isArray(value)) {
            // is a array, ensure that all array entries are a allowed type
            value.forEach(function (entry, index) {
                if (!isParameterValue(entry)) {
                    throw new TypeError("array parameter entry ".concat(key, "[").concat(index, "] was of invalid type '").concat(typeof entry, "'"));
                }
            });
        }
        else {
            // not a array, check if value is allowed type
            if (!isParameterValue(value)) {
                throw new TypeError("parameter ".concat(key, " was of invalid type '").concat(typeof value, "'"));
            }
        }
    };
    for (var key in parameters) {
        _loop_1(key);
    }
}
exports.assertParametersValid = assertParametersValid;
//# sourceMappingURL=Parameters.js.map